@@ -6,12 +6,12 @@ import sys
 import threading
 import time
 import zlib
+from abc import ABC, abstractmethod
 from contextlib import contextmanager
-from datetime import datetime
+from datetime import datetime, timezone
 from functools import wraps, partial
 
 import sentry_sdk
-from sentry_sdk._compat import text_type, utc_from_timestamp, iteritems
 from sentry_sdk.utils import (
     ContextVar,
     now,
@@ -54,8 +54,6 @@ if TYPE_CHECKING:
 
 
 _in_metrics = ContextVar("in_metrics", default=False)
-_sanitize_key = partial(re.compile(r"[^a-zA-Z0-9_/.-]+").sub, "_")
-_sanitize_value = partial(re.compile(r"[^\w\d\s_:/@\.{}\[\]$-]+", re.UNICODE).sub, "")
 _set = set  # set is shadowed below
 
 GOOD_TRANSACTION_SOURCES = frozenset(
@@ -67,6 +65,25 @@ GOOD_TRANSACTION_SOURCES = frozenset(
     ]
 )
 
+_sanitize_unit = partial(re.compile(r"[^a-zA-Z0-9_]+").sub, "")
+_sanitize_metric_key = partial(re.compile(r"[^a-zA-Z0-9_\-.]+").sub, "_")
+_sanitize_tag_key = partial(re.compile(r"[^a-zA-Z0-9_\-.\/]+").sub, "")
+
+
+def _sanitize_tag_value(value):
+    # type: (str) -> str
+    table = str.maketrans(
+        {
+            "\n": "\\n",
+            "\r": "\\r",
+            "\t": "\\t",
+            "\\": "\\\\",
+            "|": "\\u{7c}",
+            ",": "\\u{2c}",
+        }
+    )
+    return value.translate(table)
+
 
 def get_code_location(stacklevel):
     # type: (int) -> Optional[Dict[str, Any]]
@@ -108,23 +125,29 @@ def metrics_noop(func):
     return new_func
 
 
-class Metric(object):
+class Metric(ABC):
     __slots__ = ()
 
+    @abstractmethod
+    def __init__(self, first):
+        # type: (MetricValue) -> None
+        pass
+
     @property
+    @abstractmethod
     def weight(self):
-        # type: (...) -> int
-        raise NotImplementedError()
+        # type: () -> int
+        pass
 
-    def add(
-        self, value  # type: MetricValue
-    ):
-        # type: (...) -> None
-        raise NotImplementedError()
+    @abstractmethod
+    def add(self, value):
+        # type: (MetricValue) -> None
+        pass
 
+    @abstractmethod
     def serialize_value(self):
-        # type: (...) -> Iterable[FlushedMetricValue]
-        raise NotImplementedError()
+        # type: () -> Iterable[FlushedMetricValue]
+        pass
 
 
 class CounterMetric(Metric):
@@ -261,15 +284,16 @@ def _encode_metrics(flushable_buckets):
     out = io.BytesIO()
     _write = out.write
 
-    # Note on sanetization: we intentionally sanetize in emission (serialization)
+    # Note on sanitization: we intentionally sanitize in emission (serialization)
     # and not during aggregation for performance reasons.  This means that the
     # envelope can in fact have duplicate buckets stored.  This is acceptable for
     # relay side emission and should not happen commonly.
 
     for timestamp, buckets in flushable_buckets:
-        for bucket_key, metric in iteritems(buckets):
+        for bucket_key, metric in buckets.items():
             metric_type, metric_name, metric_unit, metric_tags = bucket_key
-            metric_name = _sanitize_key(metric_name)
+            metric_name = _sanitize_metric_key(metric_name)
+            metric_unit = _sanitize_unit(metric_unit)
             _write(metric_name.encode("utf-8"))
             _write(b"@")
             _write(metric_unit.encode("utf-8"))
@@ -285,7 +309,7 @@ def _encode_metrics(flushable_buckets):
                 _write(b"|#")
                 first = True
                 for tag_key, tag_value in metric_tags:
-                    tag_key = _sanitize_key(tag_key)
+                    tag_key = _sanitize_tag_key(tag_key)
                     if not tag_key:
                         continue
                     if first:
@@ -294,7 +318,7 @@ def _encode_metrics(flushable_buckets):
                         _write(b",")
                     _write(tag_key.encode("utf-8"))
                     _write(b":")
-                    _write(_sanitize_value(tag_value).encode("utf-8"))
+                    _write(_sanitize_tag_value(tag_value).encode("utf-8"))
 
             _write(b"|T")
             _write(str(timestamp).encode("ascii"))
@@ -309,7 +333,9 @@ def _encode_locations(timestamp, code_locations):
 
     for key, loc in code_locations:
         metric_type, name, unit = key
-        mri = "{}:{}@{}".format(metric_type, _sanitize_key(name), unit)
+        mri = "{}:{}@{}".format(
+            metric_type, _sanitize_metric_key(name), _sanitize_unit(unit)
+        )
 
         loc["type"] = "location"
         mapping.setdefault(mri, []).append(loc)
@@ -322,7 +348,7 @@ METRIC_TYPES = {
     "g": GaugeMetric,
     "d": DistributionMetric,
     "s": SetMetric,
-}
+}  # type: dict[MetricType, type[Metric]]
 
 # some of these are dumb
 TIMING_FUNCTIONS = {
@@ -337,7 +363,7 @@ TIMING_FUNCTIONS = {
 }
 
 
-class LocalAggregator(object):
+class LocalAggregator:
     __slots__ = ("_measurements",)
 
     def __init__(self):
@@ -391,7 +417,7 @@ class LocalAggregator(object):
         return rv
 
 
-class MetricsAggregator(object):
+class MetricsAggregator:
     ROLLUP_IN_SECONDS = 10.0
     MAX_WEIGHT = 100000
     FLUSHER_SLEEP_TIME = 5.0
@@ -485,14 +511,14 @@ class MetricsAggregator(object):
                 self._force_flush = False
             else:
                 flushable_buckets = []
-                for buckets_timestamp, buckets in iteritems(self.buckets):
+                for buckets_timestamp, buckets in self.buckets.items():
                     # If the timestamp of the bucket is newer that the rollup we want to skip it.
                     if buckets_timestamp <= cutoff:
                         flushable_buckets.append((buckets_timestamp, buckets))
 
                 # We will clear the elements while holding the lock, in order to avoid requesting it downstream again.
                 for buckets_timestamp, buckets in flushable_buckets:
-                    for _, metric in iteritems(buckets):
+                    for metric in buckets.values():
                         weight_to_remove += metric.weight
                     del self.buckets[buckets_timestamp]
 
@@ -557,6 +583,8 @@ class MetricsAggregator(object):
         # Given the new weight we consider whether we want to force flush.
         self._consider_force_flush()
 
+        # For sets, we only record that a value has been added to the set but not which one.
+        # See develop docs: https://develop.sentry.dev/sdk/metrics/#sets
         if local_aggregator is not None:
             local_value = float(added if ty == "s" else value)
             local_aggregator.add(ty, key, local_value, unit, serialized_tags)
@@ -575,7 +603,7 @@ class MetricsAggregator(object):
         if timestamp is None:
             timestamp = time.time()
         meta_key = (ty, key, unit)
-        start_of_day = utc_from_timestamp(timestamp).replace(
+        start_of_day = datetime.fromtimestamp(timestamp, timezone.utc).replace(
             hour=0, minute=0, second=0, microsecond=0, tzinfo=None
         )
         start_of_day = int(to_timestamp(start_of_day))
@@ -591,7 +619,7 @@ class MetricsAggregator(object):
                 )
 
     @metrics_noop
-    def need_code_loation(
+    def need_code_location(
         self,
         ty,  # type: MetricType
         key,  # type: str
@@ -602,7 +630,7 @@ class MetricsAggregator(object):
         if self._enable_code_locations:
             return False
         meta_key = (ty, key, unit)
-        start_of_day = utc_from_timestamp(timestamp).replace(
+        start_of_day = datetime.fromtimestamp(timestamp, timezone.utc).replace(
             hour=0, minute=0, second=0, microsecond=0, tzinfo=None
         )
         start_of_day = int(to_timestamp(start_of_day))
@@ -643,7 +671,7 @@ class MetricsAggregator(object):
             encoded_metrics = _encode_metrics(flushable_buckets)
             envelope.add_item(Item(payload=encoded_metrics, type="statsd"))
 
-        for timestamp, locations in iteritems(code_locations):
+        for timestamp, locations in code_locations.items():
             encoded_locations = _encode_locations(timestamp, locations)
             envelope.add_item(Item(payload=encoded_locations, type="metric_meta"))
 
@@ -661,14 +689,14 @@ def _serialize_tags(
         return ()
 
     rv = []
-    for key, value in iteritems(tags):
+    for key, value in tags.items():
         # If the value is a collection, we want to flatten it.
         if isinstance(value, (list, tuple)):
             for inner_value in value:
                 if inner_value is not None:
-                    rv.append((key, text_type(inner_value)))
+                    rv.append((key, str(inner_value)))
         elif value is not None:
-            rv.append((key, text_type(value)))
+            rv.append((key, str(value)))
 
     # It's very important to sort the tags in order to obtain the
     # same bucket key.
@@ -701,20 +729,18 @@ def _get_aggregator():
     )
 
 
-def _get_aggregator_and_update_tags(key, tags):
-    # type: (str, Optional[MetricTags]) -> Tuple[Optional[MetricsAggregator], Optional[LocalAggregator], Optional[MetricTags]]
+def _get_aggregator_and_update_tags(key, value, unit, tags):
+    # type: (str, Optional[MetricValue], MeasurementUnit, Optional[MetricTags]) -> Tuple[Optional[MetricsAggregator], Optional[LocalAggregator], Optional[MetricTags]]
     hub = sentry_sdk.Hub.current
     client = hub.client
     if client is None or client.metrics_aggregator is None:
         return None, None, tags
 
-    experiments = client.options.get("_experiments", {})
-
     updated_tags = dict(tags or ())  # type: Dict[str, MetricTagValue]
     updated_tags.setdefault("release", client.options["release"])
     updated_tags.setdefault("environment", client.options["environment"])
 
-    scope = hub.scope
+    scope = sentry_sdk.Scope.get_current_scope()
     local_aggregator = None
 
     # We go with the low-level API here to access transaction information as
@@ -725,31 +751,20 @@ def _get_aggregator_and_update_tags(key, tags):
         if transaction_name:
             updated_tags.setdefault("transaction", transaction_name)
         if scope._span is not None:
-            sample_rate = experiments.get("metrics_summary_sample_rate")
-            # We default the sample rate of metrics summaries to 1.0 only when the sample rate is `None` since we
-            # want to honor the user's decision if they pass a valid float.
-            if sample_rate is None:
-                sample_rate = 1.0
-            should_summarize_metric_callback = experiments.get(
-                "should_summarize_metric"
-            )
-            if random.random() < sample_rate and (
-                should_summarize_metric_callback is None
-                or should_summarize_metric_callback(key, updated_tags)
-            ):
-                local_aggregator = scope._span._get_local_aggregator()
+            local_aggregator = scope._span._get_local_aggregator()
 
+    experiments = client.options.get("_experiments", {})
     before_emit_callback = experiments.get("before_emit_metric")
     if before_emit_callback is not None:
         with recursion_protection() as in_metrics:
             if not in_metrics:
-                if not before_emit_callback(key, updated_tags):
+                if not before_emit_callback(key, value, unit, updated_tags):
                     return None, None, updated_tags
 
     return client.metrics_aggregator, local_aggregator, updated_tags
 
 
-def incr(
+def increment(
     key,  # type: str
     value=1.0,  # type: float
     unit="none",  # type: MeasurementUnit
@@ -759,14 +774,20 @@ def incr(
 ):
     # type: (...) -> None
     """Increments a counter."""
-    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(key, tags)
+    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
+        key, value, unit, tags
+    )
     if aggregator is not None:
         aggregator.add(
             "c", key, value, unit, tags, timestamp, local_aggregator, stacklevel
         )
 
 
-class _Timing(object):
+# alias as incr is relatively common in python
+incr = increment
+
+
+class _Timing:
     def __init__(
         self,
         key,  # type: str
@@ -816,7 +837,10 @@ class _Timing(object):
         # type: (Any, Any, Any) -> None
         assert self._span, "did not enter"
         aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
-            self.key, self.tags
+            self.key,
+            self.value,
+            self.unit,
+            self.tags,
         )
         if aggregator is not None:
             elapsed = TIMING_FUNCTIONS[self.unit]() - self.entered  # type: ignore
@@ -871,7 +895,9 @@ def timing(
     - it can be used as a decorator
     """
     if value is not None:
-        aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(key, tags)
+        aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
+            key, value, unit, tags
+        )
         if aggregator is not None:
             aggregator.add(
                 "d", key, value, unit, tags, timestamp, local_aggregator, stacklevel
@@ -889,7 +915,9 @@ def distribution(
 ):
     # type: (...) -> None
     """Emits a distribution."""
-    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(key, tags)
+    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
+        key, value, unit, tags
+    )
     if aggregator is not None:
         aggregator.add(
             "d", key, value, unit, tags, timestamp, local_aggregator, stacklevel
@@ -898,7 +926,7 @@ def distribution(
 
 def set(
     key,  # type: str
-    value,  # type: MetricValue
+    value,  # type: Union[int, str]
     unit="none",  # type: MeasurementUnit
     tags=None,  # type: Optional[MetricTags]
     timestamp=None,  # type: Optional[Union[float, datetime]]
@@ -906,7 +934,9 @@ def set(
 ):
     # type: (...) -> None
     """Emits a set."""
-    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(key, tags)
+    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
+        key, value, unit, tags
+    )
     if aggregator is not None:
         aggregator.add(
             "s", key, value, unit, tags, timestamp, local_aggregator, stacklevel
@@ -923,7 +953,9 @@ def gauge(
 ):
     # type: (...) -> None
     """Emits a gauge."""
-    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(key, tags)
+    aggregator, local_aggregator, tags = _get_aggregator_and_update_tags(
+        key, value, unit, tags
+    )
     if aggregator is not None:
         aggregator.add(
             "g", key, value, unit, tags, timestamp, local_aggregator, stacklevel
